@@ -94,7 +94,7 @@ def create_compute_environment(args):
         if args.ecs_container_instance_ami:
             ecs_ami_id = args.ecs_container_instance_ami
         elif args.ecs_container_instance_ami_tags:
-            ecs_ami_id = resolve_ami(**args.ecs_container_instance_ami_tags)
+            ecs_ami_id = resolve_ami(tags=args.ecs_container_instance_ami_tags)
         else:
             ecs_ami_id = get_ssm_parameter("/aws/service/ecs/optimized-ami/amazon-linux-2/recommended/image_id")
         launch_template = ensure_launch_template(ImageId=ecs_ami_id,
@@ -297,6 +297,9 @@ def add_job_defn_args(parser):
     parser.add_argument("--memory-mb", dest="memory", type=int,
                         help="Memory to allocate to the Docker container (this is both a soft and a hard limit)")
     parser.add_argument("--user", help="Name or ID of user to use in the Docker container")
+    parser.add_argument("--log-driver", help="Custom log driver")
+    parser.add_argument("--log-options", help="Custom log configuration options", nargs="+", metavar="NAME=VALUE",
+                        type=lambda x: x.split("=", 1), default=[])
 
 add_command_args(submit_parser)
 
@@ -381,35 +384,54 @@ def print_event(event):
     print(str(Timestamp(event["timestamp"])) + " " + event["message"])
 
 def get_logs(args, print_event_fn=print_event):
-    for event in CloudwatchLogReader(args.log_stream_name, head=args.head, tail=args.tail):
+    for event in CloudwatchLogReader(log_group_name=args.log_group_name,
+                                     log_stream_name=args.log_stream_name,
+                                     head=args.head,
+                                     tail=args.tail):
         print_event_fn(event)
 
 def watch(args, print_event_fn=print_event):
     job_desc = get_job_desc(args.job_id)
     args.job_name = job_desc["jobName"]
     logger.info("Watching job %s (%s)", args.job_id, args.job_name)
-    last_status, log_reader = None, None
-    while last_status not in {"SUCCEEDED", "FAILED"}:
+    last_status, log_reader, job_done = None, None, False
+    while True:
         job_desc = get_job_desc(args.job_id)
         if job_desc["status"] != last_status:
             logger.info("Job %s %s", args.job_id, format_job_status(job_desc["status"]))
             last_status = job_desc["status"]
             if job_desc["status"] in {"RUNNING", "SUCCEEDED", "FAILED"}:
                 logger.info("Job %s log stream: %s", args.job_id, job_desc.get("container", {}).get("logStreamName"))
+        container_desc = job_desc.get("container", {})
         if job_desc["status"] in {"RUNNING", "SUCCEEDED", "FAILED"}:
+            try:
+                log_group_name = container_desc["logConfiguration"]["options"]["awslogs-group"]
+            except KeyError:
+                log_group_name = "/aws/batch/job"
             if "logStreamName" in job_desc.get("container", {}):
                 args.log_stream_name = job_desc["container"]["logStreamName"]
                 if log_reader is None:
-                    log_reader = CloudwatchLogReader(args.log_stream_name, head=args.head, tail=args.tail)
+                    log_reader = CloudwatchLogReader(log_group_name=log_group_name,
+                                                     log_stream_name=args.log_stream_name,
+                                                     head=args.head,
+                                                     tail=args.tail)
                 for event in log_reader:
                     print_event_fn(event)
         if "statusReason" in job_desc:
             logger.info("Job %s: %s", args.job_id, job_desc["statusReason"])
-        if job_desc.get("container", {}).get("exitCode"):
-            return SystemExit(job_desc["container"]["exitCode"])
+        # When a job is finished, we do one last iteration to read any log lines that were still being delivered.
+        if job_done:
+            if job_desc.get("container", {}).get("exitCode"):
+                return SystemExit(job_desc["container"]["exitCode"])
+            elif last_status == "FAILED":
+                return SystemExit(-1)
+            return SystemExit(os.EX_OK)
+        if last_status in {"SUCCEEDED", "FAILED"}:
+            job_done = True
         time.sleep(1)
 
 get_logs_parser = register_parser(get_logs, parent=batch_parser, help="Retrieve logs for a Batch job")
+get_logs_parser.add_argument("log_group_name", default="/aws/batch/job")
 get_logs_parser.add_argument("log_stream_name")
 watch_parser = register_parser(watch, parent=batch_parser, help="Monitor a running Batch job and stream its logs")
 watch_parser.add_argument("job_id")
