@@ -140,10 +140,11 @@ def run(args):
             expect_task_defn["cpu"] = str(int(expect_task_defn["cpu"][:-len(" vCPU")]) * 1024)
         if expect_task_defn["memory"].endswith("GB"):
             expect_task_defn["memory"] = str(int(expect_task_defn["memory"][:-len("GB")]) * 1024)
-        assert task_defn == expect_task_defn
-        logger.debug("Reusing task definition %s", task_desc["taskDefinitionArn"])
+        for key in expect_task_defn:
+            assert expect_task_defn[key] == task_defn[key]
+        logger.info("Reusing task definition %s", task_desc["taskDefinitionArn"])
     except (ClientError, AssertionError):
-        logger.debug("Registering new ECS task definition %s", task_defn_name)
+        logger.info("Registering new ECS task definition %s", task_defn_name)
         task_desc = clients.ecs.register_task_definition(family=task_defn_name, **expect_task_defn)["taskDefinition"]
 
     network_config = {
@@ -220,8 +221,10 @@ def format_task_status(status):
 def watch(args):
     logger.info("Watching task %s (%s)", args.task_id, args.cluster)
     last_status, events_received = None, 0
-    log_reader = CloudwatchLogReader("/".join([args.task_name, args.task_name, os.path.basename(args.task_id)]),
-                                     log_group_name=args.task_name)
+    log_reader = CloudwatchLogReader(
+        log_group_name=args.task_name,
+        log_stream_name="/".join([args.task_name, args.task_name, os.path.basename(args.task_id)])
+    )
     while last_status != "STOPPED":
         res = clients.ecs.describe_tasks(cluster=args.cluster, tasks=[args.task_id])
         if len(res["tasks"]) == 1:
