@@ -22,12 +22,13 @@ Run the same search, but retrieve 10 lines of context for each match:
     aegea grep STRING LOG_GROUP --start-time=-1w -C 10
 """
 
-import os, sys, json, hashlib, time, concurrent.futures
+import os, sys, json, hashlib, time
 from datetime import datetime, timedelta
 from functools import partial
+from typing import Dict
 
 from . import register_parser, logger
-from .util import Timestamp, paginate, add_time_bound_args
+from .util import Timestamp, paginate, add_time_bound_args, ThreadPoolExecutor
 from .util.compat import timestamp
 from .util.exceptions import AegeaException
 from .util.printing import page_output, tabulate
@@ -115,10 +116,10 @@ def grep(args):
                                      startTime=int(timestamp(args.start_time) * 1000),
                                      endTime=int(timestamp(args.end_time) * 1000),
                                      queryString=args.query)
-    seen_results = {}
+    seen_results = {}  # type: Dict[str, Dict]
     print_with_context = partial(print_log_event_with_context, before=args.before_context, after=args.after_context)
     try:
-        with concurrent.futures.ThreadPoolExecutor() as executor:
+        with ThreadPoolExecutor() as executor:
             while True:
                 res = clients.logs.get_query_results(queryId=query["queryId"])
                 log_record_pointers = []
