@@ -4,6 +4,7 @@ import os, sys, json, io, gzip, time, socket, hashlib, uuid
 import requests
 from warnings import warn
 from datetime import datetime, timedelta
+from ipaddress import ip_network
 from typing import List, Dict, Any
 
 import boto3, botocore.session
@@ -19,49 +20,38 @@ from . import clients, resources
 def get_ssm_parameter(name):
     return clients.ssm.get_parameter(Name=name)["Parameter"]["Value"]
 
-def locate_ami(product, region=None, channel="releases", stream="released", root_store="ssd", virt="hvm"):
+def locate_ami(distribution, release, architecture):
     """
     Examples::
-        locate_ami(product="com.ubuntu.cloud:server:20.04:amd64", channel="daily", stream="daily", region="us-west-2")
-        locate_ami(product="Amazon Linux AMI 2016.09")
+        locate_ami(distribution="Ubuntu", release="20.04", architecture="amd64")
+        locate_ami(distribution="Amazon Linux", release="2", architecture="arm64")
     """
-    if region is None:
-        region = clients.ec2.meta.region_name
-    if product.startswith("com.ubuntu.cloud"):
-        partition = "aws"
-        if region.startswith("cn-"):
-            partition = "aws-cn"
-        elif region.startswith("us-gov-"):
-            partition = "aws-govcloud"
-        if partition not in {"aws", "aws-cn", "aws-govcloud"}:
-            raise AegeaException("Unrecognized partition {}".format(partition))
-        manifest_url = "https://cloud-images.ubuntu.com/{channel}/streams/v1/com.ubuntu.cloud:{stream}:{partition}.json"
-        manifest_url = manifest_url.format(partition=partition, channel=channel, stream=stream)
-        manifest = requests.get(manifest_url).json()
-        if product not in manifest["products"]:
-            raise AegeaException("Ubuntu version {} not found in Ubuntu cloud image manifest".format(product))
-        versions = manifest["products"][product]["versions"]
-        for version in sorted(versions.keys(), reverse=True)[:8]:
-            for ami in versions[version]["items"].values():
-                if ami["crsn"] == region and ami["root_store"] == root_store and ami["virt"] == virt:
-                    logger.info("Found %s for %s", ami["id"], ":".join([product, version, region, root_store, virt]))
-                    return ami["id"]
-    elif product.startswith("Amazon Linux"):
-        filters = {"root-device-type": "ebs" if root_store == "ssd" else root_store, "virtualization-type": virt,
-                   "architecture": "x86_64", "owner-alias": "amazon", "state": "available"}
-        images = resources.ec2.images.filter(Filters=[dict(Name=k, Values=[v]) for k, v in filters.items()])
-        for image in sorted(images, key=lambda i: i.creation_date, reverse=True):
-            if root_store == "ebs" and not image.name.endswith("x86_64-gp2"):
-                continue
-            if image.name.startswith("amzn-ami-" + virt) and image.description.startswith(product):
-                return image.image_id
-    raise AegeaException("No AMI found for {} {} {} {}".format(product, region, root_store, virt))
+    if distribution == "Amazon Linux" and release == "2":
+        if architecture == "amd64":
+            architecture = "x86_64"
+        ssm_param_name = "/aws/service/ami-amazon-linux-latest/amzn2-ami-hvm-{}-gp2".format(architecture)
+        ami_id = clients.ssm.get_parameters(Names=[ssm_param_name])["Parameters"][0]["Value"]
+        logger.info("Found %s for %s %s %s", ami_id, distribution, release, architecture)
+        return ami_id
+    elif distribution == "Ubuntu":
+        if architecture == "x86_64":
+            architecture = "amd64"
+        ssm_param_name = "/aws/service/canonical/ubuntu/{product}/{release}/stable/current/{arch}/hvm/ebs-gp2/ami-id"
+        ssm_param_name = ssm_param_name.format(product="server", release=release, arch=architecture)
+        ami_id = clients.ssm.get_parameters(Names=[ssm_param_name])["Parameters"][0]["Value"]
+        logger.info("Found %s for %s %s %s", ami_id, distribution, release, architecture)
+        return ami_id
+    raise AegeaException("No AMI found for {} {} {}".format(distribution, release, architecture))
 
 def ensure_vpc():
+    """
+    If a default VPC exists in the current account/region, return it; otherwise, return the first VPC managed by aegea,
+    creating a new one (with an associated IGW, route table, and subnet per AZ) if it does not exist.
+    """
     for vpc in resources.ec2.vpcs.filter(Filters=[dict(Name="isDefault", Values=["true"])]):
         break
     else:
-        for vpc in resources.ec2.vpcs.all():
+        for vpc in resources.ec2.vpcs.filter(Filters=[dict(Name="tag:managedBy", Values=["aegea"])]):
             break
         else:
             from ... import config
@@ -72,23 +62,35 @@ def ensure_vpc():
             clients.ec2.get_waiter("vpc_available").wait(VpcIds=[vpc.id])
             vpc.modify_attribute(EnableDnsSupport=dict(Value=config.vpc.enable_dns_support))
             vpc.modify_attribute(EnableDnsHostnames=dict(Value=config.vpc.enable_dns_hostnames))
+            clients.ec2.associate_vpc_cidr_block(VpcId=vpc.id, AmazonProvidedIpv6CidrBlock=True)
+
+            ensure_subnet(vpc, assign_ipv6_cidr_blocks=True)
+
             tags = dict(Name="aegea-igw", managedBy="aegea")
             tag_spec = dict(ResourceType="internet-gateway", Tags=encode_tags(tags))
             internet_gateway = resources.ec2.create_internet_gateway(TagSpecifications=[tag_spec])
             vpc.attach_internet_gateway(InternetGatewayId=internet_gateway.id)
             for route_table in vpc.route_tables.all():
                 route_table.create_route(DestinationCidrBlock="0.0.0.0/0", GatewayId=internet_gateway.id)
+
             tags = dict(Name="aegea-eigw", managedBy="aegea")
             tag_spec = dict(ResourceType="egress-only-internet-gateway", Tags=encode_tags(tags))
-            clients.ec2.create_egress_only_internet_gateway(VpcId=vpc.id, TagSpecifications=[tag_spec])
-            ensure_subnet(vpc)
+            res = clients.ec2.create_egress_only_internet_gateway(VpcId=vpc.id, TagSpecifications=[tag_spec])
+            eigw_id = res["EgressOnlyInternetGateway"]["EgressOnlyInternetGatewayId"]
+            for route_table in vpc.route_tables.all():
+                route_table.create_route(DestinationIpv6CidrBlock="::/0", EgressOnlyInternetGatewayId=eigw_id)
     return vpc
 
 def availability_zones():
     for az in clients.ec2.describe_availability_zones()["AvailabilityZones"]:
         yield az["ZoneName"]
 
-def ensure_subnet(vpc, availability_zone=None):
+def ensure_subnet(vpc, availability_zone=None, assign_ipv6_cidr_blocks=False):
+    """
+    Find and return a subnet in the given VPC, or create one subnet per AZ and return one of them if no suitable subnet
+    is found.
+    """
+    # FIXME: if config.vpc.map_public_ip_on_launch is set, ignore subnets that don't have that attribute set
     if availability_zone is not None and availability_zone not in availability_zones():
         msg = "Unknown availability zone {} (choose from {})"
         raise AegeaException(msg.format(availability_zone, list(availability_zones())))
@@ -97,7 +99,6 @@ def ensure_subnet(vpc, availability_zone=None):
             continue
         break
     else:
-        from ipaddress import ip_network
         from ... import config
         subnet_cidrs = ip_network(str(config.vpc.cidr[ARN.get_region()])).subnets(new_prefix=config.vpc.subnet_prefix)
         subnets = {}
@@ -107,9 +108,20 @@ def ensure_subnet(vpc, availability_zone=None):
             tag_spec = dict(ResourceType="subnet", Tags=encode_tags(tags))
             subnets[az] = resources.ec2.create_subnet(VpcId=vpc.id, CidrBlock=str(subnet_cidr), AvailabilityZone=az,
                                                       TagSpecifications=[tag_spec])
+            time.sleep(1)
             clients.ec2.get_waiter("subnet_available").wait(SubnetIds=[subnets[az].id])
             clients.ec2.modify_subnet_attribute(SubnetId=subnets[az].id,
                                                 MapPublicIpOnLaunch=dict(Value=config.vpc.map_public_ip_on_launch))
+        if assign_ipv6_cidr_blocks:
+            vpc_cidr_block = vpc.ipv6_cidr_block_association_set[0]["Ipv6CidrBlock"]
+            subnet_cidr_blocks = list(ip_network(vpc_cidr_block).subnets(new_prefix=64))
+            for i, subnet in enumerate(subnets.values()):
+                logger.info("Assigning IPv6 CIDR block %s to %s", subnet_cidr_blocks[i], subnet)
+                clients.ec2.associate_subnet_cidr_block(SubnetId=subnet.id, Ipv6CidrBlock=str(subnet_cidr_blocks[i]))
+                clients.ec2.modify_subnet_attribute(
+                    SubnetId=subnet.id,
+                    AssignIpv6AddressOnCreation=dict(Value=config.vpc.assign_ipv6_address_on_creation)
+                )
         subnet = subnets[availability_zone] if availability_zone is not None else list(subnets.values())[0]
     return subnet
 
@@ -229,16 +241,10 @@ class ARN:
             try:
                 user = resources.iam.CurrentUser().user
                 cls._default_iam_username = getattr(user, "name", ARN(user.arn).resource.split("/")[-1])
-            except Exception as e:
+            except Exception:
                 try:
-                    if "Must specify userName" in str(e) or ("assumed-role" in str(e) and "botocore-session" in str(e)):
-                        cur_session = boto3.Session()._session
-                        src_profile = cur_session.full_config["profiles"][cur_session.profile]["source_profile"]
-                        src_session = boto3.Session(profile_name=src_profile)
-                        cls._default_iam_username = src_session.resource("iam").CurrentUser().user.name
-                    else:
-                        caller_arn = ARN(clients.sts.get_caller_identity()["Arn"])
-                        cls._default_iam_username = caller_arn.resource.split("/")[-1]
+                    caller_arn = ARN(clients.sts.get_caller_identity()["Arn"])
+                    cls._default_iam_username = caller_arn.resource.split("/")[-1]
                 except Exception:
                     cls._default_iam_username = "unknown"
         return cls._default_iam_username
@@ -264,6 +270,9 @@ def add_tags(resource, dry_run=False, **tags):
 def filter_by_tags(collection, **tags):
     return collection.filter(Filters=[dict(Name="tag:" + k, Values=[v]) for k, v in tags.items()])
 
+def filter_by_tag_keys(collection, *tag_keys):
+    return collection.filter(Filters=[dict(Name="tag-key", Values=[k]) for k in tag_keys])
+
 def resolve_instance_id(name):
     filter_name = "dns-name" if name.startswith("ec2") and name.endswith("compute.amazonaws.com") else "tag:Name"
     if name.startswith("i-"):
@@ -274,13 +283,21 @@ def resolve_instance_id(name):
     except IndexError:
         raise AegeaException('Could not resolve "{}" to a known instance'.format(name))
 
-def get_bdm(max_devices=12, ebs_storage=frozenset()):
+def get_bdm(ami=None, max_devices=12, ebs_storage=frozenset()):
     # Note: d2.8xl and hs1.8xl have 24 devices
     bdm = [dict(VirtualName="ephemeral" + str(i), DeviceName="xvd" + chr(ord("b") + i)) for i in range(max_devices)]
     ebs_bdm = []  # type: List[Dict]
     for i, (mountpoint, size_gb) in enumerate(ebs_storage):
-        ebs_spec = dict(Encrypted=True, DeleteOnTermination=True, VolumeType="st1", VolumeSize=int(size_gb))
-        ebs_bdm.insert(0, dict(DeviceName="xvd" + chr(ord("z") - i), Ebs=ebs_spec))
+        if mountpoint == "/":
+            rootfs_bdm = resources.ec2.Image(ami).block_device_mappings[0]
+            if "Ebs" not in rootfs_bdm or "SnapshotId" not in rootfs_bdm.get("Ebs", []):
+                raise AegeaException("Unable to locate block device mapping for root volume of {}".format(ami))
+            rootfs_bdm["Ebs"]["VolumeSize"] = int(size_gb)
+            bdm.insert(0, rootfs_bdm)
+        else:
+            volume_type = "st1" if int(size_gb) > 125 else "gp3"
+            ebs_spec = dict(Encrypted=True, DeleteOnTermination=True, VolumeType=volume_type, VolumeSize=int(size_gb))
+            ebs_bdm.insert(0, dict(DeviceName="xvd" + chr(ord("z") - i), Ebs=ebs_spec))
     bdm.extend(ebs_bdm)
     return bdm
 
@@ -298,7 +315,7 @@ def expect_error_codes(exception, *codes):
     if getattr(exception, "response", None) and exception.response.get("Error", {}).get("Code", {}) not in codes:
         raise
 
-def resolve_ami(ami=None, **tags):
+def resolve_ami(ami=None, arch="x86_64", tags=frozenset(), tag_keys=frozenset()):
     """
     Find an AMI by ID, name, or tags.
     - If an ID is given, it is returned with no validation; otherwise, selects the most recent AMI from:
@@ -307,14 +324,19 @@ def resolve_ami(ami=None, **tags):
     - If no AMIs found, all available AMIs in this account (filtered by tags given).
     Return the AMI with the most recent creation date.
     """
+    assert arch in {"x86_64", "arm64"}
+
     if ami is None or not ami.startswith("ami-"):
         if ami is None:
-            filters = dict(Owners=["self"], Filters=[dict(Name="state", Values=["available"])])
+            filters = dict(Owners=["self"],
+                           Filters=[dict(Name="state", Values=["available"]), dict(Name="architecture", Values=[arch])])
         else:
             filters = dict(Owners=["self"], Filters=[dict(Name="name", Values=[ami])])
         all_amis = resources.ec2.images.filter(**filters)
         if tags:
             all_amis = filter_by_tags(all_amis, **tags)
+        if tag_keys:
+            all_amis = filter_by_tag_keys(all_amis, *tag_keys)
 
         current_user_amis = all_amis.filter(Filters=[dict(Name="tag:Owner", Values=[ARN.get_iam_username()])])
         amis = sorted(current_user_amis, key=lambda x: x.creation_date)
