@@ -252,11 +252,10 @@ class TestAegea(unittest.TestCase):
         make_waiter(clients.efs.describe_file_systems, "FileSystems[].LifeCycleState", "available", "pathAny")
 
     def test_locate_ami(self):
-        self.assertTrue(locate_ami("com.ubuntu.cloud:server:16.04:amd64", "us-east-1").startswith("ami-"))
-        ami = locate_ami(product="com.ubuntu.cloud:server:16.04:amd64", channel="releases", stream="released",
-                         region="us-west-2")
-        self.assertTrue(ami.startswith("ami-"))
-        self.assertTrue(locate_ami("Amazon Linux AMI 2016.09").startswith("ami-"))
+        self.assertTrue(locate_ami("Ubuntu", "20.04", "amd64").startswith("ami-"))
+        self.assertTrue(locate_ami("Ubuntu", "20.04", "arm64").startswith("ami-"))
+        self.assertTrue(locate_ami("Amazon Linux", "2", "x86_64").startswith("ami-"))
+        self.assertTrue(locate_ami("Amazon Linux", "2", "arm64").startswith("ami-"))
 
     def test_ip_ranges(self):
         get_public_ip_ranges()
